;**************************************************************************
;
;
; LCD.inc	LCD code to drive an epson SED1200
;	
; Description: 	Drives EPSON EA-C20017AR based upon Epson 
;		SED1200F if buildLCDsed1200 defined
;
;		Drives Hitachi HD44780 based Modules if if buildLCD_hd44780 defined
;
;		Common Procedures:-
;			LCDwrtStr	- display a String into 10 byte window starting at 0x80.
;			LCDdispByte	- display byte as two hex nibbles
;			LCDwrtChar	- write Char to module
;			LCDwrtCmd	- write CMD  to module
;
;			Cursor addres 0x80 is line 1 on both modules
;			Cursor addres 0xC0 is line 2 on both modules.
;
;			On SED1200 line 1 is left 10 Chars
;			On SED1200 line 2 is Right10 Chars
;
;			Treat the two sides separately, as two lines
;			
;			On HD557840 line 1 is top line
;			On HD557840 line 2 is bottom line
;	
;			LCDstart	- Initilizes Module
;
;Define one of these to select the module in main file.
;
; buildLCDsed1200
; buildLCD_HD44780
;
;
; Author: 	Doug Rice
;
;
;           
;
;**************************************************************************




;--------------------------------------------------------------------------
; Sec 3.3	LCD routines stuff 
;--------------------------------------------------------------------------

		cblock
LCDtemp		;
LCDcnt		;
LCDnibble	;
LCDPortB	; image of port B for unused bits
LCDeraseCnt	;
LCDstrPtr 	;
		endc


;--------------------------------------------------------------------------
; Sec 3.3.4.2	LCD procedures LCD string write displayes
;--------------------------------------------------------------------------
  ifdef buildLCDgetByte
LCDgetByte
	; we should have set up PCLATH
	; When LCDstrPtr gets to 0xFF
	MOVLW	high $
	MOVWF	PCLATH 
	MOVFW	LCDstrPtr;
	ADDWF	PCL,f

LCDstrs
LCDstrLTime	equ	$ - LCDstrs
	DT	"Hello"
	GOTO	LCDgetByteEnd4		; now print out 4 blanks


LCDstrLblank	equ	$ - LCDstrs
	DT	"   "
	GOTO	LCDgetByteEnd6

LCDstrLOff	equ	$ - LCDstrs
	DT	"OFF"
	GOTO	LCDgetByteEnd6

LCDstrLOn	equ	$ - LCDstrs
	DT	"ON "
	GOTO	LCDgetByteEnd6
LCDstrLOnp	equ	$ - LCDstrs
	DT	"ON+"
	GOTO	LCDgetByteEnd6


LCDLastStr

	IF  0 !=  ( high ( LCDLastStr ) - high ( LCDgetByte ) )
	Error "String Table straddles Page Boundary "
	ENDIF
	; Jump into these to select how many spaces printed
	; at the end, so that 9 chars are printed.
LCDgetByteEnd0
	DECF	LCDeraseCnt,f
LCDgetByteEnd1
	DECF	LCDeraseCnt,f
LCDgetByteEnd2
	DECF	LCDeraseCnt,f
LCDgetByteEnd3
	DECF	LCDeraseCnt,f
LCDgetByteEnd4
	DECF	LCDeraseCnt,f
LCDgetByteEnd5
	DECF	LCDeraseCnt,f
LCDgetByteEnd6
	CLRF	LCDstrPtr
	DECF	LCDstrPtr,f
	RETURN
  endif ; buildLCDgetByte

;--------------------------------------------------------------------------
; Sec 3.3.4.3	LCD procedures LCD string write displayes
;--------------------------------------------------------------------------

LCDwrtStr	; W contains the string offset
	MOVWF	LCDstrPtr;
	
	; 	Erase upto 6 chars at end of string
	MOVLW	0x06
	MOVWF	LCDeraseCnt

	; position cursor at left on line 1.
	; erase old display
	MOVLW	LCDCurLeft0	; 0x80
	CALL 	LCDwrtCmd
	GOTO	LCDwrtStr1

LCDwrtStr0
	CALL 	LCDwrtChar
LCDwrtStr1
	CALL	LCDgetByte
	INCFSZ	LCDstrPtr,f
	GOTO	LCDwrtStr0
	GOTO 	LCDwrtStrFlush

; write 10 chars to blank rest of right
LCDwrtStrFlush
	; 	W contains the number of Chars to flush
LCDwrtStrLoop
	DECFSZ	LCDeraseCnt,f
	GOTO	LCDwrtStrLoop1
	RETURN
LCDwrtStrLoop1
	MOVLW	0x20
	CALL 	LCDwrtChar
	GOTO	LCDwrtStrLoop	



;--------------------------------------------------------------------------
; Sec 3.3.4	LCD procedures
;--------------------------------------------------------------------------

;--------------------------------------------------------------------------
; Sec 3.3.4.1	LCD procedures - Multi entry display procedure for commands, nibbles,bytes,chars
;--------------------------------------------------------------------------


LCDdispByte
	MOVWF	LCDnibble

	; Display top nipple
	SWAPF	LCDnibble,w
        CALL    LCDwrtDecNibble
	; Display bottom nipple
	MOVFW   LCDnibble
        CALL    LCDwrtDecNibble
	RETURN

LCDwrtCmd
	BCF	LCDPortB,LCDbitA0
	GOTO	LCDwrtByte

LCDwrtDecNibble	; currently only displays 0..9, A..F
	ANDLW	0x0F
	ADDLW	0x06	; is it A..F, if so trigger a digit overflow
	SKPNDC
	ADDLW	7; subtract 10, then add 'A'-'0'
	ADDLW	0x30-6	; Subtract extra 6 added to cause DC
LCDwrtChar
	BSF	LCDPortB,LCDbitA0



  ifdef buildLCD_HD44780

;You don't need to read the busy line on the display.
;
;PORT B		Display		LCD Pin
;DB7		DB7		14
;DB6		DB6		13
;DB5		DB5		12
;DB4		DB4		11
;DB3		E		 6
;		R/W	gnd	 5
;DB2		RS		 4
;		V0	Vlcd	 3
;		Vdd     +5  	 2
;		Vss	0	 1

;R/W	==X====================x====
;RS	____________________________	0 for CMD, 1 for DATA
;E	____/----\____/-----\_______
;DB	==X=========X==========X====
;	      MSN	LSN

; define bits - these need to be defined for the HD44780 chip
;LCDbitCS        EQU     1	; PORTB:1
;LCDbitWR        EQU	3	; PORTB:3
;LCDbitCLK       EQU	3	; PORTB:3 or OSC Out

LCDbitE		EQU	3
LCDbitRS	EQU	2

LCDbitA0	EQU	LCDbitRS	; RS - PORTB:2 ; sometimes used for bit number

LCDCurOff       EQU     0x0C	; DisplayOn, Cursor Off, Blink Off
LCDCurOn	EQU     0x0F	; DisplayOn, Cursor On, Blink On

LCDCurLeft0     EQU     0x80	; display 80,..,89,C0..C9 ; top
LCDCurRight0    EQU     0xC0	; left



LCDwrtByte
; assume that the A0 line is correctly set,
; Byte to be displayed is in W
; SED1200 LCD chip
; CS    ----________------------------------------------------
; WR,CLK-----__--__------_-_-_-_--_-_-_-_--_-_-_-_--_-_-_-_---
; DB,A0 ---x====x====x----------------------------------------

; HD44780 LCD chip
;R/W	==X====================x====
;RS	____________________________	0 for CMD, 1 for DATA
;E	____/----\____/-----\_______
;DB	==X=========X==========X====
;	      MSN	LSN


; Assumes:- 
; LCD_DB0..3 connected to PORTB:4..7
; R/~w connected to Gnd.
; R/S,E  are equated, but connected to PORTB
; on exit PORTB<4:7> are left as inputs, so that they can
; be used for the keyboard/buttons.

        MOVWF   LCDtemp
	; set up PORTB<0:3> to Outputs,PORTB<4:7> to Outputs	
	MOVLW	0x00		; This is only required if
	MOVWF	PORTB		; The MS nibble is used as
	BSF	STATUS, RP0	; inputs when reading buttons.
	MOVLW	0x01		;
	MOVWF	TRISB		;
	BCF	STATUS, RP0	;

	BTFSC	LCDPortB, LCDbitA0 ; set RS line early
	BSF	PORTB,LCDbitA0	
	BTFSS	LCDPortB, LCDbitA0 ; set
	BCF	PORTB,LCDbitA0	

; set up to write top nibble, then pulse WR
	MOVFW	LCDtemp
	ANDLW	0xF0 	; assumes DB0..DB3 is MSB
	BTFSC	LCDPortB, LCDbitA0 ; set
	IORLW	 1 << LCDbitRS 
        MOVWF   PORTB

        BSF     PORTB,LCDbitE
	NOP
        BCF     PORTB,LCDbitE

; set up to write bottom nibble, then pulse WR
	SWAPF	LCDtemp,w
	ANDLW	0xF0 	; assumes DB0..DB3 is MSB
	BTFSC	LCDPortB, LCDbitA0 ; set
	IORLW	 1 << LCDbitRS 
        MOVWF   PORTB

        BSF     PORTB,LCDbitE
	NOP
        BCF     PORTB,LCDbitE
	NOP

	; set up PORTB<0:3> to Outputs,PORTB<4:7> to Inputs
	BSF	STATUS, RP0	;
	MOVLW	0xF1		;
	MOVWF	TRISB		;
	BCF	STATUS, RP0	;

; Run a little delay to allow the instructions to take place.
	MOVLW	20
	MOVLW	1

	MOVWF	LCDtemp
; Hold CS high and pulse WR low at least 16 times
; Now pulse CLK low 16 times to execute command
LCDloop

;        BCF     PORTB,LCDbitCLK
	NOP
	NOP
;        BSF     PORTB,LCDbitCLK
        DECFSZ  LCDtemp,f
        GOTO    LCDloop

        RETURN


;--------------------------------------------------------------------------
; Sec 3.3.4.2	LCD procedures - Initilization
;--------------------------------------------------------------------------

LCDstart

; Minimum reset instructions:

; System Reset,Line Select(2),Display On
; set up for LCD commands

;     wlcdCmd($33);
;     wlcdCmd($32);
;     wlcdCmd($28);	{ 4 bit, 2 line, 5x7 display }
;     wlcdCmd($01);     { Clear Display }
;     wlcdCmd($0F);     { $0F - Display On,CursorOn,Blink }
;     wlcdCmd($C9);     { set dd address }


        MOVLW   0x33		; force to 8 bit mode, from any mode
        CALL    LCDwrtCmd
        MOVLW   0x32		; force to 4 bit mode 
        CALL    LCDwrtCmd
        MOVLW   0x28		; 4 bit, 2 line, 5x7 display 
        CALL    LCDwrtCmd
        MOVLW   0x01		; Clear display
        CALL    LCDwrtCmd

        MOVLW   0x0F		; Display On,Cursor On,Blink
        CALL    LCDwrtCmd
        MOVLW   0x80

        MOVLW   10
        MOVWF   LCDcnt
	; Now actually display something
	MOVLW	LCDstrLTime
	CALL	LCDwrtStr
	RETURN


;To know wwhich nibble is being sent initilize to 8 bit then
;go into 4 bit mode and keep track of each nibble sent
;
;If using 4 bit interface only use the DB4 to DB7 pins.
;The 8 bits comands to convert to 4 bits mode do not care
;about the 4 LSB.
;
;With R/W =0 and RS=0:
;
;0x01	;clear display
;0x02	;return home
;0x04	;EntryModeSet
;  0x01	; S	1 = shift
;  0x02  ; I/D	1= increment/0=decrement
;0x08	;Display ON/OFF
;  0x01	; B	Blink
;  0x02	; C	Cursor On
;  0x04	; D	Display On
;0x10	;Cursor and Display Shift
;  0x04	; s/c	Shift Display/ 0=shift Cursor
;  0x08	; r/l	Shift Right,0=shift left
;0x20	;Function Set
;  0x04	; F	5x10 dots /5x7dots
;  0x08	; N	2lines	/1lines
;  0x10	; DL	8bits interface /4 bits interface.
;0x40	;Set CG RAM address
;0x80	;Set DD RAM address;

;With R/W =0 and RS=1:
;write data to CG or DD 
;With R/W =1 and RS=1:
;Read data from CG or DD 
;
;Send the commands below to Initilize.
;First mop up any 4 bit commands, and convert to 8 bit,
;either from 4 bit mode or 8 bit mode.
;Then from 8 bit node move to  4 bit mode.
;Then issue 4 bit commands.

;     wlcdCmd($33);
;     wlcdCmd($32);
;     wlcdCmd($28);
;     wlcdCmd($01);      Clear Display 
;     wlcdCmd($0F);      $0F - Display On,CursorOn,Blink 
;     wlcdCmd($C9);      set dd address 

;You don't need to read the busy line on the display.
;
;PORT B		Display		LCD Pin
;DB7		DB7		14
;DB6		DB6		13
;DB5		DB5		12
;DB4		DB4		11
;DB3		E		 6
;		R/W	gnd	 5
;DB2		RS		 4
;		V0	Vlcd	 3
;		Vdd     +5  	 2
;		Vss	0	 1


  endif


  ifdef buildLCDsed1200

;
;
;================================================================
;this needs to be changed
;
;

;--------------------------------------------------------------------------
; Sec 3.3.1	LCD introduction - SED1200F
;--------------------------------------------------------------------------

;LCD Information
;==================
;
;Having brought a number of displays for a bargin price, and finding that the 
;information provided was inadequate, this document summarizes what has been 
;found out.
;
;
;Epson EA-C20017AR
;==================
;
;20 char by one line.
;
;The display appears to be based on an SED1200F controller.
;
;Pin Connections        when looking at display, contacts on left.
;
;       1       +5V     2       Gnd        |------------------|
;       3       Vlcd    4       Clk        |                  |
;       5       -CS     6       A0         |    DISPLAY 20x1  |
;       7       -WR     8       -RD        |      -           |
;       9       DB3     10      DB2        |                  |
;       11      DB1     12      DB0        |------------------
;
; Display is 20 x 1
; Suggested values for single line mode , 10 x 1 repeated 10 x 1.
;
; + 5V   Pin 1
;    |  
;   [ ]   10k ohm
;    |  
;Vlcd    Pin 3
;    |
;   [ ]   3k ohm
;    |
;  GND  
;
;
; For proper operation run the display as two line mode.
;
;
; Initialization. leaves random characters.
;
;       CMD, System Reset       0,0x10
;       CMD, DISPLAY ON         0,0x0D
;       CMD, Line Select-On     0,0x13
;       CMD, Cursor Home        0,0x80
;       DATA,0x20 

;--------------------------------------------------------------------------
; Sec 3.3.2	LCD command equates
;--------------------------------------------------------------------------

;
; SED 1200F config commands A0=0
;
LCDsetCurDirInc EQU     0x04
LCDsetCurDirDec EQU     0x05
LCDCurInc       EQU     0x06
LCDCurDec       EQU     0x07
LCDCurBlock     EQU     0x09
LCDCurUL	EQU     0x08
LCDCurConst     EQU     0x0A
LCDCurBlink     EQU     0x0B
LCDCurDispOff   EQU     0x0C
LCDCurDispOn    EQU     0x0D
LCDCurOff       EQU     0x0E
LCDCurOn	EQU     0x0F
LCDSysReset     EQU     0x10
LCD1line	EQU     0x12
LCD2line	EQU     0x13
LCDCurX0Y0      EQU     0x80	; display has two halfs of 10 chars each
LCDCurX1Y0      EQU     0xC0	;
LCDCurLeft0     EQU     0x80	; display 80,..,89,C0..C9
LCDCurRight0    EQU     0xC0	;

LCDCgRamAddr    EQU     0x20
LCDCgRamData    EQU     0x40

;--------------------------------------------------------------------------
; Sec 3.3.3	LCD control line / hardware to port mapping equates
;--------------------------------------------------------------------------


; define binary Mask for bits
LCD_CS          EQU     2	; PORTB:1
LCD_WR          EQU	8	; PORTB:3
LCD_CLK         EQU	8	; PORTB:3 or OSC Out
LCD_A0		EQU	4	; PORTB:2 ; sometimes used for bit number
; define bits
LCDbitCS        EQU     1	; PORTB:1
LCDbitWR        EQU	3	; PORTB:3
LCDbitCLK       EQU	3	; PORTB:3 or OSC Out
LCDbitA0	EQU	2	; PORTB:2 ; sometimes used for bit number



LCDwrtByte
; assume that the A0 line is correctly set,
; Byte to be displayed is in W
;
; CS    ----________------------------------------------------
; WR,CLK-----__--__------_-_-_-_--_-_-_-_--_-_-_-_--_-_-_-_---
; DB,A0 ---x====x====x----------------------------------------


; Assumes:- 
; LCD_DB0..3 connected to PORTB:4..7
; RD connected to +5
; WR,CS,A0,CLK are equated, but connected to PORTB
; on exit PORTB<4:7> are left as inputs, so that they can
; be used for the keyboard/buttons.

        MOVWF   LCDtemp
	; set up PORTB<0:3> to Outputs,PORTB<4:7> to Outputs	
	MOVLW	0xFF		; This is only required if
	MOVWF	PORTB		; The MS nibble is used as
	BSF	STATUS, RP0	; inputs when reading buttons.
	MOVLW	0x01		;
	MOVWF	TRISB		;
	BCF	STATUS, RP0	;

	BTFSC	LCDPortB, LCDbitA0 ; set A0 line early
	BSF	PORTB,LCDbitA0	
	BTFSS	LCDPortB, LCDbitA0 ; set
	BCF	PORTB,LCDbitA0	

; set up to write top nibble, then pulse WR
	MOVFW	LCDtemp
	IORLW	0x0F 	; -CS,-A0,-WR
	BTFSS	LCDPortB, LCDbitA0 ; set
	ANDLW	~LCD_A0
        MOVWF   PORTB
        BCF     PORTB,LCDbitCS

        BCF     PORTB,LCDbitWR
	NOP
        BSF     PORTB,LCDbitWR

; set up to write bottom nibble, then pulse WR
	SWAPF	LCDtemp,w
	IORLW	0x0F 	; -CS,-A0,-WR
	BTFSS	LCDPortB, LCDbitA0 ; set
	ANDLW	~(LCD_A0 | LCD_CS)

	ANDLW	~LCD_CS
        MOVWF   PORTB
        BCF     PORTB,LCDbitWR
	NOP
	NOP

        BSF     PORTB,LCDbitWR
	NOP
	NOP
        BSF     PORTB,LCDbitCS

	; set up PORTB<0:3> to Outputs,PORTB<4:7> to Inputs
	BSF	STATUS, RP0	;
	MOVLW	0xF1		;
	MOVWF	TRISB		;
	BCF	STATUS, RP0	;

	MOVLW	20
	MOVLW	1

	MOVWF	LCDtemp
; Hold CS high and pulse WR low at least 16 times
; Now pulse CLK low 16 times to execute command
LCDloop

        BCF     PORTB,LCDbitCLK
	NOP
	NOP
        BSF     PORTB,LCDbitCLK
        DECFSZ  LCDtemp,f
        GOTO    LCDloop

	; set up PORTB<0:3> to Outputs,PORTB<4:7> to Inputs
	BSF	STATUS, RP0	;
	MOVLW	0xF1		;
	MOVWF	TRISB		;
	BCF	STATUS, RP0	;
        RETURN


;--------------------------------------------------------------------------
; Sec 3.3.4.2	LCD procedures - Initilization
;--------------------------------------------------------------------------

LCDstart

; Minimum reset instructions:

; System Reset,Line Select(2),Display On
; set up for LCD commands
;       CMD, System Reset       0,0x10
;       CMD, DISPLAY ON         0,0x0D
;       CMD, Line Select 2
;       CMD, Cursor On          0,0x0F
;       CMD, Cursor Dir         0,0x04
;       CMD, Cursor Blink off   0,0x0A
;       CMD, Cursor Home        0,0x80
;       DATA,0x20       BCF     PORTA,LCDA0

   	; LCD system reset
        MOVLW   0x10
        CALL    LCDwrtCmd

   	; LCD Display On
        MOVLW   0x0D
        CALL    LCDwrtCmd

        ; LCD Set 2 Line Mode
        MOVLW   0x13
        CALL    LCDwrtCmd

        ; LCD Display On
        MOVLW   0x0D
        CALL    LCDwrtCmd

        ; LCD Cursor Home
        MOVLW   0x80
        CALL    LCDwrtCmd

        MOVLW   10
        MOVWF   LCDcnt
	; now write some characters
LCDloop1
        MOVLW   0x20
        CALL    LCDwrtChar
        DECFSZ  LCDcnt,f
        GOTO    LCDloop1
                
        ; LCD Cursor Home

        MOVLW   0xC0
        CALL    LCDwrtCmd

        MOVLW   10
        MOVWF   LCDcnt
LCDloop2
        MOVLW   0x20
        CALL    LCDwrtChar
        DECFSZ	LCDcnt,f
        GOTO    LCDloop2
LCDloop2A
	; Now actually display something
	MOVLW	LCDstrLTime
	CALL	LCDwrtStr
	RETURN

  endif ;buildLCDsed1200

;
;
;=======================================================================
;
;
;: Symbols that need to be defined differently for different LCD chips


;RT003.ASM 390 : (LCDstart)
;LCD.INC 151   : (LCDbitA0) ; check the sense
;LCD.INC 152   : (LCDwrtByte)

;RT003.ASM 803 : (LCDCurOn)
;RT003.ASM 812 : (LCDCurOff)
;RT003.ASM 680 : (LCDCurRight0)
;LCD.INC 104   : (LCDCurLeft0)


;--------------------------------------------------------------------------
; 	END OF FILE
;--------------------------------------------------------------------------

